% This is material illustrating the methods from the book
% Financial Modelling  - Theory, Implementation and Practice with Matlab
% source
% Wiley Finance Series
% ISBN 978-0-470-74489-5
%
% Date: 02.05.2012
%
% Authors:  Joerg Kienitz
%           Daniel Wetterau
%
% Please send comments, suggestions, bugs, code etc. to
% kienitzwetterau_FinModelling@gmx.de
%
% (C) Joerg Kienitz, Daniel Wetterau
% 
% Since this piece of code is distributed via the mathworks file-exchange
% it is covered by the BSD license 
%
% This code is being provided solely for information and general 
% illustrative purposes. The authors will not be responsible for the 
% consequences of reliance upon using the code or for numbers produced 
% from using the code. 



function y = BinTree_A(S0, K, r, T, sigma, n, type)
% prices an american Asian option using a binomial tree
% S0: Spot value
% K: Strike
% r: riskless rate
% T: Maturity
% sigma: volatility
% n: periods for tree
% type: 0 put, 1 call

[S, u, d, df, p] = createbintree(S0, T, n, r, sigma); % create tree
y = asian_bintree(S, df, p, K, type);                 % price asian

end
